@@ -26,7 +26,10 @@
 
 
 #define EXTERNAL_TYPE_WINDOW_X11 (external_window_x11_get_type ())
-G_DECLARE_FINAL_TYPE (ExternalWindowX11, external_window_x11,
-		      EXTERNAL, WINDOW_X11, ExternalWindow)
+#define EXTERNAL_WINDOW_X11(object) (G_TYPE_CHECK_INSTANCE_CAST (object, EXTERNAL_TYPE_WINDOW_X11, ExternalWindowX11))
 
+typedef struct _ExternalWindowX11 ExternalWindowX11;
+typedef struct _ExternalWindowX11Class ExternalWindowX11Class;
+
+GType external_window_get_type (void);
 ExternalWindowX11 *external_window_x11_new (const char *handle_str);
