@@ -57,6 +57,8 @@ typedef struct {
 
   GSList *files;
 
+  GtkFileFilter *filter;
+
   int response;
   GSList *uris;
 
@@ -132,10 +134,15 @@ send_response (FileDialogHandle *handle)
   GVariantBuilder uri_builder;
   GVariantBuilder opt_builder;
   GSList *l;
+  const char *method_name;
+
+  method_name = g_dbus_method_invocation_get_method_name (handle->invocation);
 
   g_variant_builder_init (&opt_builder, G_VARIANT_TYPE_VARDICT);
 
-  if (handle->action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER && handle->uris)
+  if (strcmp (method_name, "SaveFiles") == 0 &&
+      handle->action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER &&
+      handle->uris)
     {
       g_autoptr(GFile) base_dir = g_file_new_for_uri (handle->uris->data);
 
@@ -174,6 +181,11 @@ send_response (FileDialogHandle *handle)
       g_variant_builder_add (&uri_builder, "s", l->data);
     }
 
+  if (handle->filter) {
+    GVariant *current_filter_variant = gtk_file_filter_to_gvariant (handle->filter);
+    g_variant_builder_add (&opt_builder, "{sv}", "current_filter", current_filter_variant);
+  }
+
   g_variant_builder_add (&opt_builder, "{sv}", "uris", g_variant_builder_end (&uri_builder));
   g_variant_builder_add (&opt_builder, "{sv}", "writable", g_variant_new_boolean (handle->allow_write));
 
@@ -182,21 +194,23 @@ send_response (FileDialogHandle *handle)
   if (handle->request->exported)
     request_unexport (handle->request);
 
-  if (handle->action == GTK_FILE_CHOOSER_ACTION_OPEN)
+  if (strcmp (method_name, "OpenFile") == 0)
     xdp_impl_file_chooser_complete_open_file (handle->impl,
                                               handle->invocation,
                                               handle->response,
                                               g_variant_builder_end (&opt_builder));
-  else if (handle->action == GTK_FILE_CHOOSER_ACTION_SAVE)
+  else if (strcmp (method_name, "SaveFile") == 0)
     xdp_impl_file_chooser_complete_save_file (handle->impl,
                                               handle->invocation,
                                               handle->response,
                                               g_variant_builder_end (&opt_builder));
-  else
+  else if (strcmp (method_name, "SaveFiles") == 0)
     xdp_impl_file_chooser_complete_save_files (handle->impl,
                                                handle->invocation,
                                                handle->response,
                                                g_variant_builder_end (&opt_builder));
+  else
+    g_assert_not_reached ();
 
   file_dialog_handle_close (handle);
 }
@@ -215,16 +229,19 @@ file_chooser_response (GtkWidget *widget,
       /* Fall through */
     case GTK_RESPONSE_DELETE_EVENT:
       handle->response = 2;
+      handle->filter = NULL;
       handle->uris = NULL;
       break;
 
     case GTK_RESPONSE_CANCEL:
       handle->response = 1;
+      handle->filter = NULL;
       handle->uris = NULL;
       break;
 
     case GTK_RESPONSE_OK:
       handle->response = 0;
+      handle->filter = gtk_file_chooser_get_filter (GTK_FILE_CHOOSER(widget));
       handle->uris = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (widget));
       break;
     }
@@ -394,6 +411,7 @@ handle_open (XdpImplFileChooser *object,
   const gchar *sender;
   GtkFileChooserAction action;
   gboolean multiple;
+  gboolean directory;
   gboolean modal;
   GdkDisplay *display;
   GdkScreen *screen;
@@ -428,9 +446,11 @@ handle_open (XdpImplFileChooser *object,
     }
   else
     {
-      action = GTK_FILE_CHOOSER_ACTION_OPEN;
       if (!g_variant_lookup (arg_options, "multiple", "b", &multiple))
         multiple = FALSE;
+      if (!g_variant_lookup (arg_options, "directory", "b", &directory))
+        directory = FALSE;
+      action = directory ? GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER : GTK_FILE_CHOOSER_ACTION_OPEN;
     }
 
   if (!g_variant_lookup (arg_options, "modal", "b", &modal))
@@ -439,7 +459,7 @@ handle_open (XdpImplFileChooser *object,
   if (!g_variant_lookup (arg_options, "accept_label", "&s", &accept_label))
     {
       if (strcmp (method_name, "OpenFile") == 0)
-        accept_label = _("_Open");
+        accept_label = multiple ? _("_Open") : _("_Select");
       else
         accept_label = _("_Save");
     }
@@ -479,6 +499,7 @@ handle_open (XdpImplFileChooser *object,
   g_object_set (preview, "margin", 10, NULL);
   gtk_widget_show (preview);
   gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (dialog), preview);
+  gtk_file_chooser_set_preview_widget_active (GTK_FILE_CHOOSER (dialog), FALSE);
   gtk_file_chooser_set_use_preview_label (GTK_FILE_CHOOSER (dialog), FALSE);
   g_signal_connect (dialog, "update-preview", G_CALLBACK (update_preview_cb), preview);
 
